import orjson
import webcolors as wc
import xarray as xr
import numpy as np
from matplotlib.colors import rgb2hex, hex2color
from pydeck.bindings.layer import Layer
from pydeck.types.base import PydeckType
from pydeck.bindings.json_tools import JSONMixin, IGNORE_KEYS, lower_camel_case_keys

import pydeck


pydeck.settings.custom_libraries = [
    {
        "libraryName": "DeckOverlayLayers",
        # "resourceUri": "https://assets.oceanum.io/packages/deck-gl-overlays/bundle.umd.cjs",
        "resourceUri": "file:///home/dave/Dev/oceanum/deck-gl-overlays/dist/bundle.umd.cjs",
    }
]


def parse_color(color):
    if isinstance(color, str):
        if color.startswith("#"):
            color = wc.hex_to_rgb(color)
        else:
            color = wc.name_to_rgb(color)
    return color


class LabelLayerException(Exception):
    pass


class LabelLayer(Layer):
    """pydeck layer with label overlays"""

    def __init__(
        self,
        data,
        id=None,
        get_label="label",
        get_color=None,
        get_background_color=[100, 100, 100, 200],
        get_border_color=[50, 50, 50, 200],
        get_border_width=1,
        get_line_width=1,
        get_icon_url=None,
        font_size=14,
        font_family="'Arial'",
        font_weight=500,
        padding=2,
        opacity=1.0,
        max_labels=20,
        line_width=1,
        offscreen_labels=None,
    ):
        """Configures a deck.gl layer for rendering data labels on a map.

        Args:
            data : geopandas.GeoDataFrame
                Data for labels with geometry column defining the label location
            id : str, default None
                Unique name for layer
            get_label : str, default 'label'
                Column name for label text
            get_color : str or array, default None
                Column name for label color or RGBA(0-255)  array for constant color
            get_background_color : str or array, default [100, 100, 100, 200]
                Column name for label background color or RGBA(0-255) array for constant color
            get_border_color : str or array, default [50, 50, 50, 200]
                Column name for label border color or RGBA(0-255) array for constant color
            get_border_width : int or str, default 1
                Column name for label border width or constant width
            get_line_width : int or str, default 1
                Column name for label line width or constant width
            get_icon_url : str, default None
                Column name for label icon url or constant icon URL
            font_size : int, default 14
                Font size in pixels
            font_family : str, default 'Arial'
                Font family
            font_weight : int, default 500
                Font weight
            padding : int, default 2
                Padding in pixels
            opacity : float, default 1.0
                Layer opacity
            max_labels : int, default 20
                Maximum number of labels to render on screen for any view
            line_width : int, default 1
                Line width for label anchor line
            offscreen_labels : str, default None
                Where to placelabels that are offscreen. One of None, 'left', 'right'



        Raises:
            LabelLayerException
                missing or invalid arguments
        """

        if isinstance(get_color, str):
            data["_r"], data["_g"], data["_b"] = zip(*data[get_color].map(parse_color))
            colorfn = ["_r", "_g", "_b", 255]
        elif isinstance(get_color, list):
            colorfn = get_color
        else:
            colorfn = [0, 0, 0, 255]

        super().__init__(
            "LabelLayer",
            data,
            id,
            use_binary_transport=False,
            pickable=True,
            get_label=get_label,
            get_color=colorfn,
            get_background_color=get_background_color,
            background=get_background_color is not None,
            get_border_color=get_border_color,
            get_border_width=get_border_width,
            get_line_width=get_line_width,
            get_icon_url=get_icon_url,
            font_size=font_size,
            font_family=font_family,
            font_weight=font_weight,
            padding=padding,
            max_labels=max_labels,
            line_width=line_width,
            offscreen_labels=offscreen_labels,
            opacity=opacity,
        )



# -*- coding: utf-8 -*-

"""Top-level package for pydeck-overlays."""

__author__ = """Oceanum Developers"""
__email__ = "developers@oceanum.science"
__version__ = "0.1.0"

from .labels import LabelLayer



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import pydeck_overlays
from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["pydeck", "webcolors"]

setup_requirements = [
    "pytest-runner",
]

test_requirements = [
    "pytest",
]

setup(
    author="Oceanum Developers",
    author_email="developers@oceanum.science",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Pydeck layers for map overlays",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=False,
    keywords="pydeck",
    name="pydeck-overlays",
    packages=find_packages(exclude=["tests", "docs"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/oceanum-io/pydeck_overlays",
    version=pydeck_overlays.__version__,
    zip_safe=False,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pydeck_overlays` package."""
import os
import tempfile
import pytest
import pydeck as pdk
import geopandas as gpd


from pydeck_overlays import LabelLayer


HERE = os.path.dirname(__file__)


@pytest.fixture
def data():
    data = gpd.read_file(os.path.join(HERE, "data", "label_test.json"))
    return data


@pytest.fixture
def view(data):
    view = pdk.ViewState(
        longitude=float(data.longitude.mean()),
        latitude=float(data.latitude.mean()),
        zoom=3,
        min_zoom=2,
        max_zoom=10,
        pitch=0,
        bearing=0,
    )
    return view


def test_label_simple(data, view):
    layer = LabelLayer(data, id="test", get_border_color=[255, 255, 255, 255])
    assert isinstance(layer, pdk.Layer)
    r = pdk.Deck(
        layer,
        initial_view_state=view,
    )

    fname = tempfile.mktemp(suffix=".html")
    r.to_html(fname, True)


def test_label_icon(data, view):
    layer = LabelLayer(
        data,
        font_size=48,
        id="test_icon",
        get_icon_url="https://img.icons8.com/color/48/000000/airplane-take-off.png",
        get_border_color=[255, 255, 255, 255],
    )
    assert isinstance(layer, pdk.Layer)
    r = pdk.Deck(
        layer,
        initial_view_state=view,
    )

    fname = tempfile.mktemp(suffix=".html")
    r.to_html(fname, True)


def test_label_colors(data, view):
    layer = LabelLayer(
        data,
        id="test_icon",
        get_color="color",
        get_background_color=[0, 0, 0, 200],
        padding=10,
    )
    assert isinstance(layer, pdk.Layer)
    r = pdk.Deck(
        layer,
        initial_view_state=view,
    )

    fname = tempfile.mktemp(suffix=".html")
    r.to_html(fname, True)
